// Morse Code stuff.
// ceptimus 2017-05-08

#ifndef MorseCode_h
#define MorseCode_h

#include <Arduino.h>
#include <avr/pgmspace.h>

#define MORSE_BUFFER_SIZE 80

class MorseCode {
  public:
    static void initialize(uint8_t buzzerPin, float WPM); // WPM is 'words per minute' according to the Morse standard.  5.0 WPM is very slow.  12.0 WPM is moderate, 20.0 WPM upwards is very fast.
    static void transmit(char *s); // transmit a string of characters.  maximum string length is limited by MORSE_BUFFER_SIZE.  Don't call unless idle is true
    static void transmit(const __FlashStringHelper *s);
    static void loop(void); // call from main program loop()  needs to be called frequently enough to allow accurate beep on/off timing.
    static bool idle; // true when class is willing to accept new transmit calls
  private:
    static const uint8_t PROGMEM Morse[];
    static char charBuffer[]; // holds queue of characters to be transmitted
    static uint16_t interval[]; // circular buffer of intervals.  even indexed intervals are 'buzzer on' milliseconds, odd indexed intervals are 'buzzer off' milliseconds
    static uint8_t readIndex; // index to circular buffer
    static uint8_t writeIndex; // index to circular buffer
    static uint8_t charIndex; // index to charBuffer
    static uint32_t prevMillis; // value of millis() from previous loop()
    static uint8_t dot_ms; // duration of a Morse code dot in milliseconds. Dashes and intervals are multiples of the dot duration according to the Morse code standard
    static uint8_t pin; // the pin number the buzzer is connected to.
    static bool done;
    static void checkCharBuffer(void);
    static void dot(void);
    static void dash(void);
    static void space(void);
    static void newInterval(uint16_t ms);
    static void complete(void);
};
#endif

